/*
  Copyright (C) 2011 Alessandro Bugatti (alessandro.bugatti@istruzione.it)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef VERY_SIMPLE_GL_H_INCLUDED
#define VERY_SIMPLE_GL_H_INCLUDED

/*! \mainpage Very Simple Graphic Library
 *
 * \section intro_sec Introduzione
 *
 * Questa è una libreria molto semplice per utilizzare la grafica
 * in C senza avere conoscenze particolari.
 * La maggior parte delle funzioni non sono altro che degli incapsulamenti
 * di funzioni QuickCG (che a loro volta sono dei wrapper intorno alla libreria SDL)
 * che vengono rese in italiano e semplificate nell'uso
 *
 */

/*! \file
 *  \brief Very Simple Graphic Library
 *  \author Alessandro Bugatti
 *  \version 0.1
 *  \date  Creazione 30/11/2011
 *  \date  Ultima modifica 17/05/2012
 */

#include <vector>
#include <SDL/SDL_mixer.h>
#include "version.h"
#include "quickcg.h"

using namespace AutoVersion;
using namespace QuickCG;
using namespace std;
#include "vsgl_types.h"
#include "oggettografico.h"
#include "rettangolo.h"
#include "cerchio.h"
#include "linea.h"
#include "sprite.h"
#include "disco.h"

extern int mouseX; ///< Variabile globale che contiene le coordinate X del mouse
extern int mouseY; ///< Variabile globale che contiene le coordinate Y del mouse
extern bool mouseTD; ///< Variabile globale che contiene lo stato del bottone destro del mouse (1 premuto, 0 no)
extern bool mouseTS; ///< Variabile globale che contiene lo stato del bottone sinistro del mouse (1 premuto, 0 no)
//extern Colore COLORE_SFONDO;
//extern vector <OggettoGrafico*> listaOggetti;
 /**
   * \brief Funzione che inizializza la grafica e deve essere chiamata
   *  prima di ogni altra.
   * \param larghezza La larghezza logica dello schermo in cui disegnare. Se l'applicazione
   * non è in fullscreen corrisponde al numero di pixel della larghezza della finestra
   * \param altezza L'altezza logica dello schermo in cui disegnare. Se l'applicazione
   * non è in fullscreen corrisponde al numero di pixel dell'altezza della finestra.
   * \param fullscreen Indica se l'applicazione deve essere eseguita in una finestra
   * (fullscreen = 0) oppure a schermo intero (fullscreen = 1)
   * \param titolo Il titolo della finestra nel caso l'applicazione non sia in fullscreen
   * altrimenti non ha effetto
   * \remarks Valori non corretti di altezza e larghezza fanno fallire l'inizializzazione
   * e di conseguenza il programma.
   */

void inizializza_grafica(int larghezza, int altezza, int fullscreen, const char* titolo);

 /**
   * \brief Disegna un punto a video di un colore scelto.
   * \param x Coordinata x del punto
   * \param y Coordinata y del punto
   * \param c Colore del punto
   */
void disegna_punto(int x, int y, const Colore& c);

/**
   * \brief Permette di generare un colore date le sue componenti rosso, verde e blu.
   * \param r Componente rossa, deve essere compresa tra 0 e 255
   * \param g Componente verde, deve essere compresa tra 0 e 255
   * \param b Componente blu, deve essere compresa tra 0 e 255
   * \return Il colore creato, che può essere usato in tutte le funzioni che richiedono un colore.
   */
Colore crea_colore(int r, int g, int b);

/**
   * \brief Permette di mostrare una scritta a video.
   * \param x Coordinata x dell'angolo in alto a sinistra della scritta
   * \param y Coordinata y dell'angolo in alto a sinistra della scritta
   * \param s La scritta da mostrare a video
   * \param c Colore della scritta: se non viene specificato il valore di default è bianco
   * \remarks Allo stato attuale le scritte sono in un unico font con dimensione 8x8.
   */
void scrivi(int x, int y, const char *s, const Colore& c = BIANCO);

/**
   * \brief Aggiorna quello che viene mostrato a schermo.
   * \remarks Deve essere chiamata ogni volta che si vuole aggiornare quello che viene mostrato
   * a video, se non viene chiamata tutte le istruzioni di disegno non hanno effetto.
   */

void aggiorna_schermo();

/**
   * \brief Mette in pausa il programma, attendendo la pressione di un tasto.
   * \remarks Non occupa tempo di CPU.
   */
void pausa();

/**
   * \brief Pulisce lo schermo, colorandolo di un solo colore
   * \param c Il colore con cui verrà ridisegnato lo schermo
   * \remarks Se non viene specificato il colore, lo schermo viene riportato
   * al colore di sfondo, che di default è nero
   * \warning Va usata con moderazione perchè è computazionalmente pesante
   */
void pulisci_schermo(const Colore& c = NESSUN_COLORE);

/**
   * \brief Imposta il colore di sfondo
   * \remarks Se non viene specificato il colore, lo sfondo viene riportato
   * al valore di default che è nero
   * \warning Per ottenere il ridisegno bisogna chiamare pulisci_schermo, questa
   * funzione imposta solo il colore ma non disegna
   */
void imposta_colore_sfondo(const Colore& c = NERO);
/**
   * \brief Crea un oggetto grafico di tipo rettangolo, con base e altezza
   * paralleli agli assi, con l'angolo in alto a sinistra in posizione (x,y)
   * e di colore specificato. Ritorna un indice che può essere poi utilizzato
   * per effettuare delle operazioni sull'oggetto
   * \param x Coordinata x dell'angolo in alto a sinistra
   * \param y Coordinata y dell'angolo in alto a sinistra
   * \param larghezza Larghezza del rettangolo
   * \param altezza Altezza del rettangolo
   * \param c Colore del rettangolo, se non specificato è bianco
   * \return Un indice che rappresenta l'oggetto e che deve essere utilizzato
   * quando si vogliono fare delle operazioni su di esso
   * \remarks Il rettangolo viene solo creato, non disegnato a video: per disegnarlo
   * é necessario usare la funzione disegna_oggetto, passando l'indice dell'oggetto
   * ottenuto da questa funzione
   */
int crea_rettangolo(int x, int y, int larghezza, int altezza, const Colore& c = BIANCO);

/**
   * \brief Crea un oggetto grafico di tipo cerchio, con il centro in posizione (x,y)
   * e di colore specificato. Ritorna un indice che può essere poi utilizzato
   * per effettuare delle operazioni sull'oggetto
   * \param x Coordinata x del centro
   * \param y Coordinata y del centro
   * \param r Raggio del cerchio
   * \param c Colore del cerchio, se non specificato è bianco
   * \return Un indice che rappresenta l'oggetto e che deve essere utilizzato
   * quando si vogliono fare delle operazioni su di esso
   * \remarks Il cerchio viene solo creato, non disegnato a video: per disegnarlo
   * é necessario usare la funzione disegna_oggetto, passando l'indice dell'oggetto
   * ottenuto da questa funzione
   */
int crea_cerchio(int x, int y, int r, const Colore& c = BIANCO);

/**
   * \brief Uguale a crea_cerchio, solo che in questo caso il cerchio ha lo sfondo
   * di colore c (disegna insomma un cerchio pieno).
   * Ritorna un indice che può essere poi utilizzato
   * per effettuare delle operazioni sull'oggetto
   * \param x Coordinata x del centro
   * \param y Coordinata y del centro
   * \param r Raggio del disco
   * \param c Colore del disco, se non specificato è bianco
   * \return Un indice che rappresenta l'oggetto e che deve essere utilizzato
   * quando si vogliono fare delle operazioni su di esso
   * \remarks Il disco viene solo creato, non disegnato a video: per disegnarlo
   * é necessario usare la funzione disegna_oggetto, passando l'indice dell'oggetto
   * ottenuto da questa funzione
   */
int crea_disco(int x, int y, int r, const Colore& c = BIANCO);

/**
   * \brief Crea un oggetto grafico di tipo linea, con un estremo in posizione
   * x1,y1 e l'altro in posizione x2,y2
   * \param x1 Coordinata x del primo estremo
   * \param y1 Coordinata y del primo estremo
   * \param x2 Coordinata x del secondo estremo
   * \param y2 Coordinata y del secondo estremo
   * \param c Colore della linea
   * \return Un indice che rappresenta l'oggetto e che deve essere utilizzato
   * quando si vogliono fare delle operazioni su di esso
   * \remarks La linea viene solo creata, non disegnata a video: per disegnarla
   * é necessario usare la funzione disegna_oggetto, passando l'indice dell'oggetto
   * ottenuto da questa funzione
   */
int crea_linea(int x1, int y1, int x2, int y2, const Colore& c = BIANCO);

/**
   * \brief Crea un oggetto grafico di tipo sprite, cioè immagine
   * in posizione x,y
   * \param x Coordinata x dell'angolo in alto a sinistra
   * \param y Coordinata y del'angolo in alto a sinistra
   * \param filename Nome del file che si vuole caricare
   * \return Un indice che rappresenta l'oggetto e che deve essere utilizzato
   * quando si vogliono fare delle operazioni su di esso
   * \remarks L'immagine viene solo creata, non disegnata a video: per disegnarla
   * é necessario usare la funzione disegna_oggetto, passando l'indice dell'oggetto
   * ottenuto da questa funzione
   * \remarks L'immagine può solo essere in formato PNG
   */
int crea_sprite(int x, int y, const string& filename);

/**
   * \brief Disegna a video un oggetto di cui viene passato l'indice come parametro
   * \param indice_oggetto L'indice (un numero intero positivo) che viene ottenuto
   * dalla funzione crea_oggetto
   * \warning Se l'indice passato non è un indice valido, la funzione non fa nulla
   * \remarks La funzione è in grado di disegnare qualsiasi tipo di oggetto grafico.
   * Per vedere gli effetti della funzione è comunque necessario chiamare la funzione
   * di aggiornamento dello schermo
   */
void disegna_oggetto(int indice_oggetto);

/**
   * \brief Disegna a video tutti gli oggetti creati
   * \remarks La funzione è in grado di disegnare qualsiasi tipo di oggetto grafico.
   * Per vedere gli effetti della funzione è comunque necessario chiamare la funzione
   * di aggiornamento dello schermo
   */
void disegna_oggetti();

/**
   * \brief Muove un oggetto sullo schermo
   * \param x Nuova coordinata x dell'angolo in alto a sinistra
   * \param y Nuova coordinata y dell'angolo in alto a sinistra
   * \param indice_oggetto L'indice (un numero intero positivo) che viene ottenuto
   * dalla funzione crea_oggetto
   * \warning Se l'indice passato non è un indice valido, la funzione non fa nulla
   * \remarks La funzione oltre che a muovere l'oggetto ne cancella anche la sua
   * copia nella posizione precedente, disegnando il colore di sfondo. Questo ovviamente
   * richiede che lo sfondo sia uniforme.
   */
void muovi_oggetto(int x, int y, int indice_oggetto);

/**
   * \brief Mette in pausa il programma per un certo tempo
   * \param millisecondi Durata della pausa in millisecondi
   * \remarks Durante una pausa il programma non è in grado di fare nient'altro
   */
void ritardo(int millisecondi);

/**
   * \brief Legge lo stato del mouse e lo memorizza nelle variabili mouseX, mouseY, mouseTD, mouseTS
   * \remarks Questa funzione deve essere chiamata quando si vuole leggere lo stato del mouse: quando
   * chiamata aggiorna i valori nelle variabili globali mouseX, mouseY, mouseTD, mouseTS
   */
void leggi_mouse();

/**
   * \brief Diventa vera quando viene premuto il tasto ESC o la x gi chiusura della finsetra, falsa altrimenti
   * \remarks Viene usata come condizione d'uscita nel main loop, per terminare il programma
   * quando l'utente preme il tasto ESC
   * \return 1 se viene premuto ESC o la X di chiusura della finestra, 0 altrimenti
   */
int fatto();

/**
   * \brief Aspetta che l'utente inserisca una stringa da tastiera, ha un comportamento simile alla gets
   * \param input La stringa dove verrà memorizzato quanto scritto dall'utente
   * \param messaggio Il messaggio che viene mostrato dove l'utente deve inserire l'input
   * \param cancella Indica se quanto scritto deve sparire dopo che viene premuto INVIO oppure rimanere.
   * \param x Coordinata x dell'angolo in alto a sinistra del messaggio
   * \param y Coordinata y dell'angolo in alto a sinistra del messaggio
   * \param c Colore del messaggio e dell'input dell'utente
   * \warning Deve essere cura del programmatore che input sia sufficientemente grande da contenere la stringa
   * inserita dall'utente
   * \warning La funzione è bloccante, quindi finchè l'utente non preme il tasto INVIO il programma
   * rimane bloccato
   */

void leggi_stringa(char* input, const char* messaggio, int cancella, int x, int y, const Colore& c = NERO);

void inizializza_audio();

void carica_musica(const char *nomefile);

void suona_musica();

void ferma_musica();

/**
   * \brief Legge lo stato della tastiera e deve essere chiamata <b>prima</b> di usare la funzione tasto_premuto
   * \remarks Questa è una modalità semplice per far leggere se un tasto è premuto o meno (nel momento in cui
   * tasto_premuto è chiamata) e può essere usata ad esempio nei videogiochi quando si vuole sparare a raffica
   * oppure durante i movimenti
   * \attention In altri casi è più interessante sapere se un tasto è stato premuto piuttosto che il suo stato attuale
   * (se non è chiara la differenza forse per voi non è importante), allo stato attuale non è prevista una funzione
   * che lo faccia
   */
void leggi_tastiera();

/**
   * \brief Verifica se un pulsante è premuto o no nel momento in cui è chiamata
   * \param tasto Il tasto come definito nella libreria SDL, per la tabella completa si rimanda a
   * http://wiki.libsdl.org/moin.cgi/SDLKeycodeLookup. Per le lettere e le cifre basta scrivere SDLK_n, dove n
   * è la lettera o la cifra che interessa
   * \return Vero se tasto è premuto, falso altrimenti.
   * \attention Deve essere chiamata dopo leggi_tastiera
   */
bool tasto_premuto(int tasto);

/**
   * \brief Copia in s il valore corrente di versione della libreria
   * \param s La stringa che conterrà la versione corrente
   * \attention Viene garantito che la dimensione massima del vettore di cartteri copiato in s è 100
   * (compreso il terminatore). E' cura del programmatore far sì che s sia sufficientemente capiente
   */
void versione(char *s);

/**
   * \brief Ritorna la quantità di millisecondi che sono passati da quando è stato avviato il programma
   * \return NUmero di millisecondi da quando è stato avviato il programma
   */
unsigned long millisecondi();

int cacca();


#endif // VERY_SIMPLE_GL_H_INCLUDED
